from maritime_schema.api import publish_schema


def test_publish_schema():
    # Prepare
    # (nothing to prepare)
    # Execute
    publish_schema()
    # Assert
    # (nothing to assert. Assertion is that no exception is thrown.)
